#################################################################################
# Created       : Martin Foster
# Modified      : 08/26/2000
#################################################################################
#
# Param - Modules part of Webchat designed to handle parameters diffrently
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################
package Ethereal::Param;

use strict;							# Strict variable enforcement
use Ethereal::Crypto;						# Cryptographic Module


#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;
	my $self  = { };
	bless $self, $class;					# Bless $self into designated class
	return $self;
}

#####################
# Cleanup
#
# This function will allow one to clear parameters in the list
# stored within this object.  Essentially it allows to remove based
# on simple keywords.  Much cleaner

sub Cleanup
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $key;						# Parameter key


	####################
	# Program area

	# Cycle and remove
	while ($key = shift(@_))
	{
		delete $self->{$key};
	}
}



#####################
# Embed Information
#
# This function will embed retreived parameters for the next pass.    By making use of
# the Cypto module the use of text mangling and unmangling is no longer needed.

sub Embed
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $cgi      = shift;					# CGI Handle
	my $choice   = shift;					# Choice of the matter

	my $crypt;
	my $string;						# Encrypted parameter string


	#####################
	# Data members

	# Retreive encrypted string
	$crypt  = new Ethereal::Crypto;
	$string = $crypt->EncryptParam($cgi, @_, 'MORI', time);

	# Determin style
	$string = ($choice < 2) ? $cgi->hidden('SEC', $string) : "\?SEC\=$string";

	# Return value
	return $string;

}

# Alias for Normal
sub EmbedNormal
{ return Embed(shift, shift, 0, @_); }

# Alias for Inline
sub EmbedInline
{ return Embed(shift, shift, 2, @_); }


#####################
# GetParam
#
# Due to the annoyance of having to retrieve parameters from the $cgi->param
# command, I have opted to move them directly into a hash.  Passable to functions
# with little or no trouble and generally avoiding having the same value 
# re-applied to the parameter, causing problems for changes and easier calling
# via a hash.

sub GetParam
{
	#####################
	# Data members	
	my $self     = shift;					# Reference to self
	my $cgi      = shift;					# Handle to CGI

	my $crypt;						# Cruptographic link
	my $param;						# Specific parameter
	my $string;						# String of parameters

	my @names;						# Array of named parameters reveived


	#####################
	# Program area

	# Security measure
	$cgi->delete('CRYPT');
	$cgi->delete('MORI');

	# Decrypt and add to hash
	if (defined($cgi->param('SEC')))
	{
		$string = $cgi->param('SEC');
		$crypt  = new Ethereal::Crypto; 

		# Create and decrypt
		$crypt->DecryptParam($cgi, $string, $self);

		# Precautionary measures
		$cgi->delete('SEC');
	}

	# Get list of all parameters received
	@names = $cgi->param();	
	
	# Cycle though parameter list
	while ($param = shift(@names))
	{
		$self->{$param} = $cgi->param($param);		# Create hash with parameters
		$cgi->delete($param);				# Delete parameter list as one goes
	}
}

1;								# Return true
