#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 03/30/2001
#################################################################################
#
# Realm Configuration - Script part of Webchat designed to add/remove realms
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Audit trail
use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table handle


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %config;							# System hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationAdmin())
	{
		# Title
		print $cgi->h1('Realm Administration'), "\n";

		# Realm additions
		if (defined($param->{'ADDITION'}))
		{ RealmAddition($database, $cgi, $param); }

		# Realms displays
		elsif (defined($param->{'REALMTYPE'}))
		{ RealmDisplay($database, $cgi, $param); }

		# Selection
	 	else 
		{ RealmType($database, $cgi, $param); }
	}

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Realm Addition
#
# Sub-routine that displays additions of new realms.

sub RealmAddition
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	
	my $table    = new Ethereal::Table($cgi);		# Tale handle

	my @radiolang   = qw(relaxed restricted);		# Language
	my @radioaccess = qw(relaxed restricted list);		# Access
	my @radioguest  = qw(yes no post);			# Guests
	my @radiotags   = qw(relaxed restricted);		# Tags
	my @radiotype	= qw(yes no hybrid);			# Realm type

	# Initialize radio buttons
	my $radiolang   = $cgi->radio_group('NEWLANG',  \@radiolang,  $radiolang[0],  'true');
	my $radioaccess = $cgi->radio_group('NEWACCESS',\@radioaccess,$radioaccess[0],'true');
	my $radioguest  = $cgi->radio_group('NEWGUEST', \@radioguest, $radioguest[0], 'true');
	my $radiotags   = $cgi->radio_group('NEWTAGS',  \@radiotags,  $radiotags[0],  'true');
	my $radiotype	= $cgi->radio_group('NEWTYPE',  \@radiotype,  $radiotype[0],  'true');

	
	#####################
	# Program area

	# Cleanup
	$param->Cleanup('ADDITION');

	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Prompt and display realm name
	$table->MakeTop();
	$table->MakeValid('<B>Realm Name</B>',   $cgi->textfield('NEWNAME'  , '', 20, 30));
	$table->MakeValid('<B>Realm Type</B>',   $radiotype);
	$table->MakeBlank();

	# Image restrictions
	$table->MakeValid('<B>Image Height</B>', $cgi->textfield('NEWHEIGHT', '0', 20, 5));
	$table->MakeValid('<B>Image Width</B>',  $cgi->textfield('NEWWIDTH' , '0', 20, 5));
	$table->MakeBlank();

	# Language selection
	$table->MakeValid('<B>Language</B>',        $radiolang);
	$table->MakeBlank();

	# Access type
	$table->MakeValid('<B>Access type</B>',     $radioaccess);
	$table->MakeBlank();

	# Guest access
	$table->MakeValid('<B>Guest Access</B>',    $radioguest);
	$table->MakeBlank();

	# Tag enforcement
	$table->MakeValid('<B>Tag enforcement</B>', $radiotags);
	$table->MakeBlank();

	# Submit and reset
	$table->MakeValid($cgi->submit('Add'), $cgi->reset('Clear'));
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";


}

#####################
# RealmDisplay
#
# Sub-routine that will display and handle basic operations reguarding direction
# of calls.   As in remove and additions.

sub RealmDisplay
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list reference

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement; 						# Query handle

	my $audit;						# Audit handle 
	my $date     = new Ethereal::Date();			# Date handler

	my $color    = 0;					# Colour changer
	my $url      = $cgi->url();				# Self referencing URL

	my $in_back;						# Back button inline
	my $in_add;						# Inline button inline
	my $values;						# Received values


	#####################
	# Program area

	# Audit handle
	$audit = new Ethereal::Audit($database, $cgi, $param);

	# Data removal
	if (defined($param->{'DELETE'}))
	{
		# Remove from satellite tables
		$data->do("DELETE FROM PuppeteerRestriction WHERE RealmName=?", {}, $param->{'DELETE'});
		$data->do("DELETE FROM PuppetInvitation WHERE RealmName=?", {}, $param->{'DELETE'});
		$data->do("DELETE FROM RealmSupervisor WHERE RealmName=?", {}, $param->{'DELETE'});
		$data->do("DELETE FROM Restriction WHERE RealmName=?", {}, $param->{'DELETE'});
		$data->do("DELETE FROM Board WHERE RealmName=?", {}, $param->{'DELETE'});

		# Delete parent table
		$data->do("DELETE FROM Realm WHERE RealmName=?", {}, $param->{'DELETE'});

		# Audit Trail
		$audit->RealmRemoval($param->{'DELETE'});

		# Cleanup
		$param->Cleanup('DELETE');
	}

	# Data addition
	if (defined( $param->{'NEWNAME'} ))
	{
		# Execute
		$data->do("INSERT INTO Realm 
			(RealmName,
			 RealmLanguage,
			 RealmAccess,
			 RealmGuests,
			 RealmPublic,
			 RealmTags,
			 RealmImageHeight,
			 RealmImageWidth,
			 RealmTimestamp)
			VALUES (?,?,?,?,?,?,?,?,?)", {}, 

		 $param->{'NEWNAME'}, 
		 $param->{'NEWLANG'},
		 $param->{'NEWACCESS'}, 
		 $param->{'NEWGUEST'},
		 $param->{'NEWTYPE'},
		 $param->{'NEWTAGS'}, 
		 $param->{'NEWHEIGHT'},
		 $param->{'NEWWIDTH'}, 
		 time);

		# Audit Trail
		$audit->RealmAddition($param->{'NEWNAME'});

		# Cleanup
		$param->Cleanup('NEWNAME');
	}

	# Links
	# Generate inline
	($in_back) = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});
	($in_add)  = $param->EmbedInline($cgi, %{$param}, 'ADDITION', 'True');

	print "<CENTER><P>",
	      "<A HREF=\"$url$in_back\">[Back]</A> ",
	      "<A HREF=\"$url$in_add\">[Addition]</A>",
	      "</P</CENTER>";

	# Data viewing
	# Prepare and execute
	$statement = $data->prepare("SELECT RealmName, 
		 RealmLanguage, 
		 RealmAccess, 
		 RealmGuests, 
		 RealmTags, 
		 RealmTimestamp 
		FROM  Realm 
		WHERE RealmPublic=? 
		ORDER BY RealmName");
	$statement->execute($param->{'REALMTYPE'});

	print "<TABLE WIDTH=\"100%\">";

	# Display header
	print "\t<TR>\n";
	print "\t\t<TD><B>Realm Name</B></TD>\n";
	print "\t\t<TD WIDTH=\"50\"><B>Language</B></TD>\n";
	print "\t\t<TD WIDTH=\"50\"><B>Access</B></TD>\n";
	print "\t\t<TD WIDTH=\"50\"><B>Guests</B></TD>\n";
	print "\t\t<TD WIDTH=\"50\"><B>Tags</B></TD>\n";
	print "\t\t<TD WIDTH=\"50\"><B>Creation</B></TD>\n";
	print "\t\t<TD></TD>\n";
	print "\t</TR>\n";

	# Fetch rows of data
	while ($values = $statement->fetchrow_hashref())
	{
		# Determin colour of row
		if ($color != 0)				# White
		{ print "\t<TR>\n"; $color = 0; }
		else						# Pale grey
		{ print "\t<TR BGCOLOR=\"\#999999\">\n";  $color = 1; }

		print "\t\t<TD><B>$values->{'RealmName'}</B></TD>\n";
		print "\t\t<TD><I>$values->{'RealmLanguage'}</I></TD>\n";
		print "\t\t<TD><I>$values->{'RealmAccess'}</I></TD>\n";
		print "\t\t<TD><I>$values->{'RealmGuests'}</I></TD>\n";
		print "\t\t<TD><I>$values->{'RealmTags'}</I></TD>\n";
		print "\t\t<TD><I>", $date->GetDate($values->{'RealmTimestamp'}), "</I></TD>\n";

		# Start form
		print "\t\t", $cgi->start_form();
		print "\t\t", $param->EmbedNormal($cgi, %{$param}, 'DELETE', $values->{'RealmName'}), "\n";

		# Submit widget
		print "\t\t<TD>\n";
		print "\t\t\t", $cgi->submit('Remove'), "\n";
		print "\t\t</TD>\n";

		# End form
		print "\t\t", $cgi->end_form(), "\n";

		print "\t</TR>\n";
	}

	$statement->finish();					# Finish SQL statement
	print "</TABLE>\n";					# End of table
}


#####################
# Realm Type
#
# Simply allows the user to determine which type of rooms to view private or 
# public based on a simple statement.

sub RealmType						# Database, CGI
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list

	my $table    = new Ethereal::Table($cgi);		# Table handler

	my $list;	
	my @list     = qw(yes no hybrid);			# List of choices


	#####################
	# Program area
		
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Prepare list
	$list = $cgi->radio_group('REALMTYPE', \@list, $list[0], 'true');

	# Prompt and Display Document choices
	$table->MakeTop();
	$table->MakeValid('<B>Public Realm</B>', $list);
	$table->MakeBlank();

	$table->MakeValid($cgi->submit('List Realms'), ' ');
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}
